/* ======================================================================
    This file is part of ffDiaporama
    ffDiaporama is a tools to make diaporama as video
    Copyright (C) 2011-2012 Dominique Levray <levray.dominique@bbox.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
   ====================================================================== */

#ifndef CCUSTOMGRAPHICSRECTITEM_H
#define CCUSTOMGRAPHICSRECTITEM_H

// Basic inclusions (common to all files)
#include "../sharedfiles/_GlobalDefines.h"

// Specific inclusions
#include "_Diaporama.h"

#include <QGraphicsRectItem>
#include <QGraphicsScene>

//======================================
// Specific defines for this dialog box
//======================================
#define TYPE_wgt_QCompositionWidget     0
#define TYPE_DlgSlideProperties         1
#define TYPE_DlgImageCorrection         2

struct  sMagneticRuler {
    bool     MagneticRuler;
    double   MagnetX1;
    double   MagnetY1;
    double   MagnetX2;
    double   MagnetY2;
    double   MagnetX3;  // Centering
    double   MagnetY3;  // Centering
};

//---------------------------------------------------------------------------------------------------------------------------------------------------------------
// class use to add interractive resize to QGraphicsRectItem object
//---------------------------------------------------------------------------------------------------------------------------------------------------------------
class cCustomGraphicsRectItem;
class cResizeGraphicsRectItem : public QGraphicsRectItem {
public:
    cCustomGraphicsRectItem *RectItem;
    int                     ZValue;
    int                     TypeItem;
    bool                    IsCapture;
    bool                    BlockRecursion;
    int                     CurrentPenWith;

    explicit cResizeGraphicsRectItem(QGraphicsScene *scene,cCustomGraphicsRectItem *RectItem,int ZValue,int TypeItem,QGraphicsItem *parent=NULL);
    virtual ~cResizeGraphicsRectItem();

    void                CalcPosition();

protected:
    virtual void        mousePressEvent(QGraphicsSceneMouseEvent *event);
    virtual void        mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    virtual QVariant    itemChange(GraphicsItemChange change,const QVariant &value);
    virtual void        paint(QPainter *painter,const QStyleOptionGraphicsItem *option,QWidget *widget=NULL);

private:
    void                ResizeUpperLeft(QPointF &newpos);
    void                ResizeUpperRight(QPointF &newpos);
    void                ResizeBottomLeft(QPointF &newpos);
    void                ResizeBottomRight(QPointF &newpos);
    void                ResizeUpper(QPointF &newpos);
    void                ResizeBottom(QPointF &newpos);
    void                ResizeLeft(QPointF &newpos);
    void                ResizeRight(QPointF &newpos);
};

//---------------------------------------------------------------------------------------------------------------------------------------------------------------
// class use to crop rectangle into the image
//---------------------------------------------------------------------------------------------------------------------------------------------------------------

class cCustomGraphicsRectItem : public QGraphicsRectItem {
public:
    QWidget             *ParentWidget;
    int                 ParentWidgetType;
    double               *x,*y,*w,*h,*zoom;
    sMagneticRuler      *MagneticRuler;     // Link to MagneticRuler parent structure
    bool                BlockZoomChange;    // flag to block zoom changing during change % to pixel
    bool                IsCapture;
    bool                KeepAspectRatio;    // true if aspect ratio is keep during resize
    double              AspectRatio;
    int                 IndexKey;
    bool                IsVisible;

    cResizeGraphicsRectItem *UpperLeft;
    cResizeGraphicsRectItem *UpperRight;
    cResizeGraphicsRectItem *BottomLeft;
    cResizeGraphicsRectItem *BottomRight;
    cResizeGraphicsRectItem *Upper;
    cResizeGraphicsRectItem *Bottom;
    cResizeGraphicsRectItem *Left;
    cResizeGraphicsRectItem *Right;

    explicit cCustomGraphicsRectItem(QGraphicsScene *scene,int ZValue,double *x,double *y,double *zoom,double *w,double *h,double xmax,double ymax,
                                     bool KeepAspectRatio,double AspectRatio,sMagneticRuler *MagneticRuler,QWidget *ParentWidget,int ParentWidgetType,
                                     int IndexKey,bool IsVisible);
    virtual ~cCustomGraphicsRectItem();

    void                SendRefreshBackgroundImage();
    void                RecalcEmbededResizeRectItem();

protected:
    virtual void        paint(QPainter *painter,const QStyleOptionGraphicsItem *option,QWidget *widget=0);
    virtual void        mousePressEvent(QGraphicsSceneMouseEvent *event);
    virtual void        mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    virtual void        mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
    virtual QVariant    itemChange(GraphicsItemChange change,const QVariant &value);
    virtual void        keyPressEvent(QKeyEvent *event);

private:
    double               StockW,StockH;  // Stock value when zoom mode is use
};

#endif // CCUSTOMGRAPHICSRECTITEM_H
